@@ -24,8 +24,8 @@
 #include "gailadjustment.h"
 
 static void	 gail_progress_bar_class_init        (GailProgressBarClass *klass);
-static void      gail_progress_bar_real_init         (GailWidget     *widget,
-                                                      GtkWidget      *gtk_widget);
+static void      gail_progress_bar_real_initialize   (AtkObject      *obj,
+                                                      gpointer       data);
 static void      gail_progress_bar_finalize          (GObject        *object);
 
 
@@ -46,7 +46,6 @@ static void      gail_progress_bar_value_changed     (GtkAdjustment  *adjustment
 
 static GailWidgetClass *parent_class = NULL;
 
-
 GType
 gail_progress_bar_get_type (void)
 {
@@ -88,17 +87,18 @@ static void
 gail_progress_bar_class_init		(GailProgressBarClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-
+  AtkObjectClass *class = ATK_OBJECT_CLASS (klass);
   GailWidgetClass *widget_class;
 
   widget_class = (GailWidgetClass*)klass;
 
   widget_class->notify_gtk = gail_progress_bar_real_notify_gtk;
-  widget_class->init = gail_progress_bar_real_init;
+
+  class->initialize = gail_progress_bar_real_initialize;
 
   gobject_class->finalize = gail_progress_bar_finalize;
 
-  parent_class = g_type_class_ref (GAIL_TYPE_WIDGET);
+  parent_class = g_type_class_peek_parent (klass);
 }
 
 AtkObject* 
@@ -111,24 +111,24 @@ gail_progress_bar_new (GtkWidget *widget)
 
   object = g_object_new (GAIL_TYPE_PROGRESS_BAR, NULL);
 
-  gail_widget_init (GAIL_WIDGET (object), widget);
-
   accessible = ATK_OBJECT (object);
+  atk_object_initialize (accessible, widget);
+
   accessible->role = ATK_ROLE_PROGRESS_BAR;
 	
   return accessible;
 }
 
 static void
-gail_progress_bar_real_init (GailWidget         *widget,
-                             GtkWidget          *gtk_widget)
+gail_progress_bar_real_initialize (AtkObject *obj,
+                                   gpointer  data)
 {
-  GailProgressBar *progress_bar = GAIL_PROGRESS_BAR (widget);
+  GailProgressBar *progress_bar = GAIL_PROGRESS_BAR (obj);
   GtkProgress *gtk_progress;
 
-  GAIL_WIDGET_CLASS (parent_class)->init (widget, gtk_widget);
+  ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
-  gtk_progress = GTK_PROGRESS (gtk_widget);
+  gtk_progress = GTK_PROGRESS (data);
   /*
    * If a GtkAdjustment already exists for the spin_button,
    * create the GailAdjustment
@@ -139,7 +139,7 @@ gail_progress_bar_real_init (GailWidget         *widget,
       g_signal_connect (gtk_progress->adjustment,
                         "value-changed",
                         G_CALLBACK (gail_progress_bar_value_changed),
-                        widget);
+                        obj);
     }
   else
     progress_bar->adjustment = NULL;
