@@ -21,6 +21,7 @@
 #include <gtk/gtk.h>
 #include "gailwidget.h"
 #include "gailnotebookpage.h"
+#include "gailcanvaswidget.h"
 
 static void gail_widget_class_init (GailWidgetClass *klass);
 
@@ -95,8 +96,8 @@ static gboolean   gail_widget_real_focus_gtk     (GtkWidget     *widget,
 static void       gail_widget_focus_event        (AtkObject     *obj,
                                                   gboolean      focus_in);
 
-static void       gail_widget_real_init          (GailWidget    *widget,
-                                                  GtkWidget     *gtk_widget);
+static void       gail_widget_real_initialize    (AtkObject     *obj,
+                                                  gpointer      data);
 static GtkWidget* gail_widget_find_viewport      (GtkWidget     *widget);
 static gboolean   gail_widget_on_screen          (GtkWidget     *widget);
 
@@ -145,9 +146,8 @@ gail_widget_class_init (GailWidgetClass *klass)
   AtkObjectClass *class = ATK_OBJECT_CLASS (klass);
   GtkAccessibleClass *accessible_class = GTK_ACCESSIBLE_CLASS (klass);
 
-  parent_class = g_type_class_ref (GTK_TYPE_ACCESSIBLE);
+  parent_class = g_type_class_peek_parent (klass);
 
-  klass->init = gail_widget_real_init;
   klass->notify_gtk = gail_widget_real_notify_gtk;
   klass->focus_gtk = gail_widget_real_focus_gtk;
 
@@ -158,28 +158,7 @@ gail_widget_class_init (GailWidgetClass *klass)
   class->get_parent = gail_widget_get_parent;
   class->ref_state_set = gail_widget_ref_state_set;
   class->get_index_in_parent = gail_widget_get_index_in_parent;
-}
-
-/**
- * gail_widget_init:
- * @widget: a #GailWidget
- * @gtk_widget: a #GtkWidget for which the GailWidget was created.
- *
- * This function is called when implementing subclasses of #GailWidget.
- * It does initialization required for the new object. It is intended 
- * that this function is called only in the gail_..._new() methods used 
- * to create an instance of a subclass of #GailWidget
- **/
-void 
-gail_widget_init (GailWidget *widget,
-                  GtkWidget  *gtk_widget)
-{
-  GailWidgetClass *klass;
-  g_return_if_fail (GAIL_IS_WIDGET (widget));
-
-  klass = GAIL_WIDGET_GET_CLASS (widget);
-  if (klass->init)
-    klass->init (widget, gtk_widget); 
+  class->initialize = gail_widget_real_initialize;
 }
 
 /**
@@ -187,27 +166,29 @@ gail_widget_init (GailWidget *widget,
  * and specifies a handler to be called when the GtkWidget is destroyed.
  **/
 static void 
-gail_widget_real_init  (GailWidget *widget,
-                        GtkWidget  *gtk_widget)
+gail_widget_real_initialize (AtkObject *obj,
+                             gpointer  data)
 {
   GtkAccessible *accessible;
-  AtkObject *obj;
+  GtkWidget *widget;
 
-  obj = ATK_OBJECT (widget);
+  g_return_if_fail (GTK_IS_WIDGET (data));
+
+  widget = GTK_WIDGET (data);
   obj->layer = ATK_LAYER_WIDGET;
 
-  accessible = GTK_ACCESSIBLE (widget);
-  accessible->widget = gtk_widget;
+  accessible = GTK_ACCESSIBLE (obj);
+  accessible->widget = widget;
   gtk_accessible_connect_widget_destroyed (accessible);
-  g_signal_connect_after (gtk_widget,
+  g_signal_connect_after (widget,
                           "focus-in-event",
                           G_CALLBACK (gail_widget_focus_gtk),
                           NULL);
-  g_signal_connect_after (gtk_widget,
+  g_signal_connect_after (widget,
                           "focus-out-event",
                           G_CALLBACK (gail_widget_focus_gtk),
                           NULL);
-  g_signal_connect (gtk_widget,
+  g_signal_connect (widget,
                     "notify",
                     G_CALLBACK (gail_widget_notify_gtk),
                     widget);
@@ -216,11 +197,11 @@ gail_widget_real_init  (GailWidget *widget,
   /*
    * Add signal handlers for GTK signals required to support property changes
    */
-  g_signal_connect (gtk_widget,
+  g_signal_connect (widget,
                     "map",
                     G_CALLBACK (gail_widget_map_gtk),
                     NULL);
-  g_signal_connect (gtk_widget,
+  g_signal_connect (widget,
                     "unmap",
                     G_CALLBACK (gail_widget_map_gtk),
                     NULL);
@@ -236,9 +217,9 @@ gail_widget_new (GtkWidget *widget)
 
   object = g_object_new (GAIL_TYPE_WIDGET, NULL);
 
-  gail_widget_init (GAIL_WIDGET (object), widget);
-
   accessible = ATK_OBJECT (object);
+  atk_object_initialize (accessible, widget);
+
   accessible->role = ATK_ROLE_UNKNOWN;
 
   return accessible;
@@ -269,6 +250,7 @@ static void
 gail_widget_destroyed (GtkWidget     *widget,
                        GtkAccessible *accessible)
 {
+  accessible->widget = NULL;
   atk_object_notify_state_change (ATK_OBJECT (accessible), ATK_STATE_DEFUNCT,
                                   TRUE); 
 }
@@ -345,6 +327,7 @@ gail_widget_get_parent (AtkObject *accessible)
          * State is defunct
          */
         return NULL;
+      g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
 
       parent_widget = widget->parent;
       if (parent_widget == NULL)
@@ -422,15 +405,10 @@ gail_widget_ref_state_set (AtkObject *accessible)
 static gint
 gail_widget_get_index_in_parent (AtkObject *accessible)
 {
-  GtkWidget *widget, *parent_widget;
-  gint index = -1;
-  GList *children, *tmp_list;
-
-  if (accessible->accessible_parent)
-    {
-      if (GAIL_IS_NOTEBOOK_PAGE (accessible->accessible_parent))
-        return 0;
-    }
+  GtkWidget *widget;
+  GtkWidget *parent_widget;
+  gint index;
+  GList *children;
 
   widget = GTK_ACCESSIBLE (accessible)->widget;
 
@@ -440,24 +418,45 @@ gail_widget_get_index_in_parent (AtkObject *accessible)
      */
     return -1;
 
+  if (accessible->accessible_parent)
+    {
+      AtkObject *parent;
+
+      parent = accessible->accessible_parent;
+
+      if (GAIL_IS_NOTEBOOK_PAGE (parent) ||
+          GAIL_IS_CANVAS_WIDGET (parent))
+        return 0;
+      else
+        {
+          gint n_children, i;
+          gboolean found = FALSE;
+
+          n_children = atk_object_get_n_accessible_children (parent);
+          for (i = 0; i < n_children; i++)
+            {
+              AtkObject *child;
+
+              child = atk_object_ref_accessible_child (parent, i);
+              if (child == accessible)
+                found = TRUE;
+
+              g_object_unref (child); 
+              if (found)
+                return i;
+            }
+        }
+    }
+
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), -1);
   parent_widget = widget->parent;
   g_return_val_if_fail (GTK_IS_CONTAINER (parent_widget), -1);
 
-  tmp_list = children = gtk_container_children (GTK_CONTAINER (parent_widget));
+  children = gtk_container_children (GTK_CONTAINER (parent_widget));
 
-  while (tmp_list)
-    {
-      if (GTK_WIDGET (tmp_list->data) == widget)
-        {
-          index++;
-          g_list_free (children);
-          return index;
-        }
-      tmp_list = g_list_next (tmp_list);
-      index++;
-    }
+  index = g_list_index (children, widget);
   g_list_free (children);
-  return -1;  
+  return index;  
 }
 
 static void 
@@ -468,8 +467,6 @@ atk_component_interface_init (AtkComponentIface *iface)
   /*
    * Use default implementation for contains and get_position
    */
-  iface->contains = NULL;
-  iface->get_position = NULL;
   iface->add_focus_handler = gail_widget_add_focus_handler;
   iface->get_extents = gail_widget_get_extents;
   iface->get_size = gail_widget_get_size;
@@ -484,12 +481,28 @@ static guint
 gail_widget_add_focus_handler (AtkComponent    *component,
                                AtkFocusHandler handler)
 {
-  return g_signal_connect_closure (component, 
-                                   "focus-event",
-                                   g_cclosure_new (
-                                   G_CALLBACK (handler), NULL,
-                                   (GClosureNotify) NULL),
-                                   FALSE);
+  GSignalMatchType match_type;
+  gulong ret;
+  guint signal_id;
+
+  match_type = G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC;
+  signal_id = g_signal_lookup ("focus-event", ATK_TYPE_OBJECT);
+
+  ret = g_signal_handler_find (component, match_type, signal_id, 0, NULL,
+                               handler, NULL);
+  if (!ret)
+    {
+      return g_signal_connect_closure_by_id (component, 
+                                             signal_id, 0,
+                                             g_cclosure_new (
+                                             G_CALLBACK (handler), NULL,
+                                             (GClosureNotify) NULL),
+                                             FALSE);
+    }
+  else
+    {
+      return 0;
+    }
 }
 
 static void 
@@ -501,7 +514,8 @@ gail_widget_get_extents (AtkComponent   *component,
                          AtkCoordType   coord_type)
 {
   GdkWindow *window;
-  gint x_parent, y_parent, x_toplevel, y_toplevel;
+  gint x_window, y_window;
+  gint x_toplevel, y_toplevel;
   GtkWidget *widget = GTK_ACCESSIBLE (component)->widget;
 
   if (widget == NULL)
@@ -510,6 +524,8 @@ gail_widget_get_extents (AtkComponent   *component,
      */
     return;
 
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+
   *width = widget->allocation.width;
   *height = widget->allocation.height;
   if (!gail_widget_on_screen (widget) || (!GTK_WIDGET_DRAWABLE (widget)))
@@ -520,19 +536,27 @@ gail_widget_get_extents (AtkComponent   *component,
     }
 
   if (widget->parent)
-    window = gtk_widget_get_parent_window (widget);
+    {
+      *x = widget->allocation.x;
+      *y = widget->allocation.y;
+      window = gtk_widget_get_parent_window (widget);
+    }
   else
-    window = widget->window;
-   
-  gdk_window_get_origin (window, &x_parent, &y_parent);
+    {
+      *x = 0;
+      *y = 0;
+      window = widget->window;
+    }
+  gdk_window_get_origin (window, &x_window, &y_window);
+  *x += x_window;
+  *y += y_window;
 
-  *x = widget->allocation.x + x_parent;
-  *y = widget->allocation.y + y_parent;
  
  if (coord_type == ATK_XY_WINDOW) 
     { 
       window = gdk_window_get_toplevel (widget->window);
       gdk_window_get_origin (window, &x_toplevel, &y_toplevel);
+
       *x -= x_toplevel;
       *y -= y_toplevel;
     }
@@ -551,6 +575,8 @@ gail_widget_get_size (AtkComponent   *component,
      */
     return;
 
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+
   *width = widget->allocation.width;
   *height = widget->allocation.height;
 }
@@ -560,7 +586,7 @@ gail_widget_grab_focus (AtkComponent   *component)
 {
   GtkWidget *widget = GTK_ACCESSIBLE (component)->widget;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
   if (GTK_WIDGET_CAN_FOCUS (widget))
     {
       gtk_widget_grab_focus (widget);
@@ -592,6 +618,7 @@ gail_widget_set_extents (AtkComponent   *component,
      * Object is defunct
      */
     return FALSE;
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
 
   if (GTK_WIDGET_TOPLEVEL (widget))
     {
@@ -635,6 +662,7 @@ gail_widget_set_position (AtkComponent   *component,
      * Object is defunct
      */
     return FALSE;
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
 
   if (GTK_WIDGET_TOPLEVEL (widget))
     {
@@ -675,6 +703,7 @@ gail_widget_set_size (AtkComponent   *component,
      * Object is defunct
      */
     return FALSE;
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
 
   if (GTK_WIDGET_TOPLEVEL (widget))
     {
@@ -822,36 +851,6 @@ gail_widget_focus_event (AtkObject   *obj,
   atk_object_notify_state_change (obj, ATK_STATE_FOCUSED, focus_in);
 }
 
-/*
- * The following function is general purpose, so it might make sense to move
- * it to a general purpose utility file at some point.
- */
-
-/**
- * gail_widget_get_origins:
- * @widget: a #GtkWidget
- * @x_widget: the x-origin of the widget->window
- * @y_widget: the y-origin of the widget->window
- * @x_window: the top-level x-origin of the widget->window
- * @y_window: the top-level y-origin of the widget->window
- *
- * Gets the origin of the widget window, and the origin of the
- * widgets top-level window.
- **/
-void
-gail_widget_get_origins (GtkWidget *widget,
-                         gint      *x_widget,
-                         gint      *y_widget,
-                         gint      *x_window,
-                         gint      *y_window)
-{
-  GdkWindow *window;
-
-  gdk_window_get_origin (widget->window, x_widget, y_widget);
-  window = gdk_window_get_toplevel (widget->window);
-  gdk_window_get_origin (window, x_window, y_window);
-}
-
 static GtkWidget*
 gail_widget_find_viewport (GtkWidget *widget)
 {
